
CREATE DOMAIN SQL_IDENTIFIER AS CHARACTER VARYING (128)
	CHARACTER SET SQL_TEXT;

CREATE DOMAIN CHARACTER_DATA AS CHARACTER VARYING (4096)
	CHARACTER SET SQL_TEXT;

CREATE DOMAIN CARDINAL_NUMBER AS INTEGER
	CONSTRAINT CARDINAL_NUMBER_DOMAIN_CHECK CHECK ( VALUE >= 0 );

CREATE TABLE INFORMATION_SCHEMA_CATALOG_NAME
(
	"CATALOG_NAME"	SQL_IDENTIFIER,

	CONSTRAINT INFORMATION_SCHEMA_CATALOG_NAME_PRIMARY_KEY
		PRIMARY KEY ( "CATALOG_NAME" )
);

INSERT INTO INFORMATION_SCHEMA_CATALOG_NAME VALUES(CURRENT_CATALOG);

CREATE VIEW CATALOGS
   AS SELECT CATALOG_PK, "CATALOG_NAME" FROM DEFINITION_SCHEMA.CATALOGS;

CREATE VIEW USERS
   AS SELECT USER_PK, USER_NAME, PASSWORD, DEFAULT_SCHEMA
	FROM DEFINITION_SCHEMA.USERS;

CREATE VIEW SCHEMATA
   AS SELECT SCHEMA_PK, SCHEMA_OWNER_PK, "SCHEMA_NAME", CATALOG_PK
	FROM DEFINITION_SCHEMA.SCHEMATA;

CREATE VIEW TABLES
   AS SELECT TABLE_PK, SCHEMA_PK, "TABLE_NAME", TABLE_TYPE
	FROM DEFINITION_SCHEMA.TABLES;

CREATE VIEW LS_INDEXES
   AS SELECT LS_INDEXES_PK, CATALOG_PK, SCHEMA_PK, INDEX_NAME, TABLE_PK,
             INDEX_ARGS, INDEX_HANDLE, VERSION, FLAGS, COLLATION_PK
       FROM DEFINITION_SCHEMA.LS_INDEXES;

CREATE VIEW DATA_TYPE_DESCRIPTOR
   AS SELECT TABLE_OR_DOMAIN_PK, COLUMN_NAME_PK, DATA_TYPE, DATA_TYPE_CODE,
             CHARACTER_MAXIMUM_LENGTH, CHARACTER_OCTET_LENGTH,
             CHARACTER_SET_PK, COLLATION_PK, NUMERIC_PRECISION,
             NUMERIC_PRECISION_RADIX, NUMERIC_SCALE, DATETIME_PRECISION,
             INTERVAL_FROM, INTERVAL_TO, DATETIME_PRECISION_LEADING,
             DATA_TYPE_DESCRIPTOR_PK
	FROM DEFINITION_SCHEMA.DATA_TYPE_DESCRIPTOR;
             
CREATE VIEW DOMAINS
   AS SELECT DOMAIN_PK, SCHEMA_PK, "DOMAIN_NAME", IS_NULLABLE
	FROM DEFINITION_SCHEMA.DOMAINS;

CREATE VIEW VIEWS
   AS SELECT VIEW_PK, TABLE_PK, CHECK_OPTION, IS_UPDATABLE
	FROM DEFINITION_SCHEMA.VIEWS;

CREATE VIEW INDEXES
   AS SELECT INDEXES_PK, CATALOG_PK, SCHEMA_PK, INDEX_NAME, TABLE_PK,
             INDEX_COLUMN_COUNT, INDEX_COLUMNS, CONSTRAINT_NAME_PK, INDEX_TYPE
         FROM DEFINITION_SCHEMA.INDEXES;

CREATE VIEW INDEX_COLUMN_USAGE
   AS SELECT INDEX_COLUMN_USAGE_PK, INDEX_PK, TABLE_PK, COLUMN_PK, ORDINAL_POSITION
         FROM DEFINITION_SCHEMA.INDEX_COLUMN_USAGE;

CREATE VIEW TABLE_CACHE_SETTINGS
	( SCHEMA_PK,
	  TABLE_PK,
	  LOWER_LIMIT,
	  UPPER_LIMIT,
	  PERCENT,
	  PERSISTENT,
	  PRELOAD )
   AS SELECT SCHEMA_PK, TABLES_PK, LOWER_LIMIT, UPPER_LIMIT, PERCENT,
             PERSISTENT, PRELOAD
	FROM DEFINITION_SCHEMA.TABLE_CACHE_SETTINGS;

CREATE VIEW CONSTRAINT_NAMES
   AS SELECT CONSTRAINT_NAMES_PK, "CONSTRAINT_NAME", CONSTRAINT_KIND
	FROM DEFINITION_SCHEMA.CONSTRAINT_NAMES;

CREATE VIEW TABLE_CONSTRAINTS
   AS SELECT TABLE_PK, CONSTRAINT_TYPE, CONSTRAINT_NAME_PK,
             IS_DEFERRABLE, INITIALLY_DEFERRED
      FROM DEFINITION_SCHEMA.TABLE_CONSTRAINTS;

CREATE VIEW KEY_COLUMN_USAGE
   AS SELECT TABLE_PK, CONSTRAINT_NAME_PK, COLUMN_PK, ORDINAL_POSITION
        FROM DEFINITION_SCHEMA.KEY_COLUMN_USAGE;

CREATE VIEW REFERENTIAL_CONSTRAINTS
   AS SELECT CONSTRAINT_NAME_PK, UNIQUE_CONSTRAINT_NAME_PK, MATCH_OPTION,
             UPDATE_RULE, DELETE_RULE
        FROM DEFINITION_SCHEMA.REFERENTIAL_CONSTRAINTS;

CREATE VIEW COLUMNS
   AS SELECT COLUMN_PK, "COLUMN_NAME", ORDINAL_POSITION,
             TABLE_PK, IS_NULLABLE, COLUMN_DEFAULT, DOMAIN_PK
        FROM DEFINITION_SCHEMA.COLUMNS;

CREATE VIEW COLUMN_PRIVILEGES
   AS SELECT GRANTOR_PK, GRANTEE_PK, TABLE_PK, COLUMN_PK,
             PRIVILEGE_TYPE, IS_GRANTABLE
	FROM DEFINITION_SCHEMA.COLUMN_PRIVILEGES;

CREATE VIEW TABLE_PRIVILEGES
   AS SELECT GRANTOR_PK, GRANTEE_PK, TABLE_PK, PRIVILEGE_TYPE, IS_GRANTABLE
	FROM DEFINITION_SCHEMA.TABLE_PRIVILEGES;

CREATE VIEW PSM_ROUTINES
   AS SELECT SCHEMA_PK, SPECIFIC_NAME, ROUTINE_NAME, PARAMETER_COUNT,
             SQL_STMT, ROUTINE_KIND, RETURN_TYPE_PK, PSM_DEFINITION_PK
        FROM DEFINITION_SCHEMA.PSM_ROUTINES;

CREATE VIEW EXECUTE_PRIVILEGES
   AS SELECT GRANTOR_PK, GRANTEE_PK, SCHEMA_PK, ROUTINE_PK, IS_GRANTABLE
	FROM DEFINITION_SCHEMA.EXECUTE_PRIVILEGES;

CREATE VIEW GROUPS
   AS SELECT GROUP_PK, GROUP_NAME FROM DEFINITION_SCHEMA.GROUPS;

CREATE VIEW GROUP_USER_USAGE
   AS SELECT GROUP_PK, USER_PK FROM DEFINITION_SCHEMA.GROUP_USER_USAGE;

CREATE TRANSLATION TO_UPPER FOR SQL_TEXT TO SQL_TEXT
	FROM EXTERNAL('ToUpper.trans');

CREATE TRANSLATION TO_LOWER FOR SQL_TEXT TO SQL_TEXT
	FROM EXTERNAL('ToLower.trans');

CREATE TABLE EOMODELER_RELATIONS(
   "SCHEMA_NAME"	CHARACTER VARYING(256),
   RELATION_NAME	CHARACTER VARYING(256),
   JOIN_SEMANTIC	INTEGER, 
   TO_MANY		INTEGER, 
   SOURCE_TABLE		CHARACTER VARYING(256),
   SOURCE_COLUMNS	CHARACTER VARYING(256), 
   DESTINATION_TABLE	CHARACTER VARYING(256),
   DESTINATION_COLUMNS	CHARACTER VARYING(256)
);

CREATE TABLE BLACK_AND_WHITE_LIST(
   BLACK_AND_WHITE_LIST_PK	INTEGER DEFAULT UNIQUE PRIMARY KEY,
   IP_ADDR			CHAR(256),
   NETMASK			CHAR(256),
   WHITE_LISTED			CHAR(3),
   SECURE			CHAR(3)
);

INSERT INTO BLACK_AND_WHITE_LIST VALUES(0, '127.000.000.001', '255.255.255.255', 'YES', 'NO');
INSERT INTO BLACK_AND_WHITE_LIST VALUES(1, '000.000.000.000', '000.000.000.000', 'YES', 'YES');

CREATE PSEUDO TABLE ROOT_ROOT(
   VERSION		VARCHAR(32),
   "CATALOG_NAME"		VARCHAR(128),
   ROOT_TABLES		INT,
   ROOT_COLUMNS		INT,
   DB_PASSWORD		VARCHAR(128),
   FLAGS			INT,
   CREATION_TIME		BIT(64),
   PREVIOUS_START		BIT(64),
   MAX_FETCH_SIZE		INT,
   TRANSACTION_LOG_FLAGS	INT,
   TRANSACTION_LOG_LIMIT	INT,
   DB_ID			BIT(96)
);

CREATE PSEUDO TABLE ROOT_TABLES(
   ROOT_TABLE_PK		INT,
   ROOT_TABLE_NAME	VARCHAR(128)
);

CREATE PSEUDO TABLE ROOT_COLUMNS(
   ROOT_COLUMN_PK		INT,
   ROOT_COLUMN_POSITION	INT,
   ROOT_TABLE_FK		INT,
   ROOT_COLUMN_NAME	VARCHAR(64),
   ROOT_COLUMN_DATATYPE	INT,
   ROOT_COLUMN_SIZE	INT
);

CREATE PSEUDO TABLE DEVICES (
  "PK"       INTEGER,
  "NAME"     CHARACTER VARYING(128),
  "PATH"     CHARACTER VARYING(1024),
  "CAPACITY" LONGINT
);
GRANT SELECT ON DEVICES TO "_SYSTEM";

CREATE PSEUDO TABLE PARTITIONS (
   "PK"            INTEGER,
   "NAME"          CHARACTER VARYING(128),
   "DEVICE_PK"     INTEGER,
   "BASE_ADDRESS"  LONGINT, 
   "CAPACITY"      LONGINT,
   "BLOCKS_IN_USE" LONGINT,
   "HIGH_WATER"    LONGINT
);
GRANT SELECT ON PARTITIONS TO "_SYSTEM";

CREATE PSEUDO TABLE DISK_ZONES (
   "PK"           INTEGER,
   "NAME"         CHARACTER VARYING(128)
);
GRANT SELECT ON DISK_ZONES TO "_SYSTEM";

CREATE PSEUDO TABLE DISK_ZONE_PARTITIONS (
   "DISK_ZONE_PK"  INTEGER,
   "PARTITION_PK"  INTEGER,
   "POSITION"      INTEGER
);
GRANT SELECT ON DISK_ZONE_PARTITIONS TO "_SYSTEM";

CREATE PSEUDO TABLE ACTIVITY_REPORT(
   "HOUR"	INTEGER,
   "MINUTE"	INTEGER,
   "SELECT"	INTEGER,
   "UPDATE"	INTEGER,
   "DELETE"	INTEGER,
   "INSERT"	INTEGER,
   "OTHER"	INTEGER
);
GRANT SELECT ON ACTIVITY_REPORT TO PUBLIC;

CREATE PSEUDO TABLE DISK_ZONE_USAGE_TABLES(
   CATALOG_PK	INTEGER,
   SCHEMA_PK	INTEGER,
   TABLE_PK	INTEGER,
   "TABLE_NAME"	VARCHAR(256),
   DZ_DATA	INTEGER,
   DZ_VARCHAR	INTEGER,
   DZ_LOB	INTEGER,
   DZ_INDEX	INTEGER
);
GRANT SELECT ON DISK_ZONE_USAGE_TABLES TO "_SYSTEM";

CREATE PSEUDO TABLE DRIVER_CACHE_STATISTICS(
   "BYTE_SIZE"           INTEGER,     -- 0
   ELEMENT_COUNT         INTEGER,     -- 1
   HASH_WIDTH            INTEGER,     -- 2
   LOW_REHASH_LIMIT      INTEGER,     -- 3
   HIGH_REHASH_LIMIT     INTEGER,     -- 4
   TIME_TO_LIVE          INTEGER,     -- 5
   HIT_FRACTION          INTEGER,     -- 6
   TIME_TO_DIE_FACTOR    INTEGER,     -- 7
   HASH_EVALUATION_COUNT LONGINT,     -- 8
   HASH_COMPARES_COUNT   LONGINT,     -- 9
   LOOKUP_COUNT          LONGINT,     -- 10
   INSERT_COUNT          LONGINT,     -- 11
   REMOVE_COUNT          LONGINT,     -- 12
   DROP_COUNT            LONGINT,     -- 13
   HIT_COUNT             LONGINT,     -- 14
   REHASH_COUNT          LONGINT,     -- 15
   CURRENT_HIT_COUNT     LONGINT,     -- 16
   CURRENT_VICTIM_COUNT  LONGINT,     -- 17
   CURRENT_LOOKUPS       LONGINT      -- 18
);
GRANT SELECT ON DRIVER_CACHE_STATISTICS TO "_SYSTEM";

CREATE PSEUDO TABLE TABLE_BASE_STATISTICS
(
   DATA_TABLE_PK	 INTEGER,
   
   OPEN_COUNT		 LONGINT,
   VERSION_COUNT	 LONGINT,
   
   "ROW_COUNT"           LONGINT,
   ROW_INSERT_COUNT      LONGINT,
   ROW_READ_COUNT        LONGINT,
   ROW_UPDATE_COUNT      LONGINT,
   ROW_DELETE_COUNT      LONGINT,
   ROW_HIT_COUNT         LONGINT,
   ROW_DROP_COUNT        LONGINT,
   
   STRING_COUNT          LONGINT,
   STRING_INSERT_COUNT   LONGINT,
   STRING_READ_COUNT     LONGINT,
   STRING_DELETE_COUNT   LONGINT,
   STRING_HIT_COUNT      LONGINT,
   STRING_DROP_COUNT     LONGINT,
   
   INDEX_COMPARE_COUNT	 LONGINT,
   INDEX_CREATE_COUNT	 LONGINT,
   INDEX_FILL_COUNT	     LONGINT,
   INDEX_READ_COUNT	     LONGINT
);
GRANT SELECT ON TABLE_BASE_STATISTICS TO "_SYSTEM";

CREATE VIEW TABLE_STATISTICS
 ( 
   "SCHEMA_NAME",
   "TABLE_NAME",
    OPEN_COUNT,
    VERSION_COUNT,
    "ROW_COUNT",
    ROW_INSERT_COUNT,
    ROW_READ_COUNT,
    ROW_UPDATE_COUNT,
    ROW_DELETE_COUNT,
    ROW_HIT_COUNT,
    ROW_DROP_COUNT,
    
    STRING_COUNT,
    STRING_INSERT_COUNT,
    STRING_READ_COUNT,
    STRING_DELETE_COUNT,
    STRING_HIT_COUNT,
    STRING_DROP_COUNT,
    
    INDEX_COMPARE_COUNT,
    INDEX_CREATE_COUNT,
    INDEX_READ_COUNT,
    INDEX_FILL_COUNT 
 ) AS 
SELECT 
    T2."SCHEMA_NAME",
    T1."TABLE_NAME",
    T0.OPEN_COUNT,
    T0.VERSION_COUNT,
    T0."ROW_COUNT",
    T0.ROW_INSERT_COUNT,
    T0.ROW_READ_COUNT,
    T0.ROW_UPDATE_COUNT,
    T0.ROW_DELETE_COUNT,
    T0.ROW_HIT_COUNT,
    T0.ROW_DROP_COUNT,
    T0.STRING_COUNT,
    T0.STRING_INSERT_COUNT,
    T0.STRING_READ_COUNT,
    T0.STRING_DELETE_COUNT,
    T0.STRING_HIT_COUNT,
    T0.STRING_DROP_COUNT,
    T0.INDEX_COMPARE_COUNT,
    T0.INDEX_CREATE_COUNT,
    T0.INDEX_FILL_COUNT,  
    T0.INDEX_READ_COUNT
FROM 
    INFORMATION_SCHEMA.TABLE_BASE_STATISTICS T0,
    DEFINITION_SCHEMA.TABLES T1,
    DEFINITION_SCHEMA.SCHEMATA T2
WHERE
   T1.DATA_TABLE_PK = T0.DATA_TABLE_PK AND
   T2.SCHEMA_PK     = T1.SCHEMA_PK;


CREATE COLLATION CASE_INSENSITIVE
   FOR SQL_TEXT
   FROM EXTERNAL('CaseInsensitive.coll1');
GRANT USAGE ON COLLATION CASE_INSENSITIVE TO PUBLIC;

CREATE INDEX ON EOMODELER_RELATIONS("SCHEMA_NAME", SOURCE_TABLE);

GRANT SELECT, UPDATE, INSERT, DELETE ON EOMODELER_RELATIONS TO PUBLIC;
GRANT SELECT ON INFORMATION_SCHEMA_CATALOG_NAME TO PUBLIC;

GRANT SELECT ON BLACK_AND_WHITE_LIST TO PUBLIC;
GRANT SELECT, UPDATE, INSERT, DELETE ON BLACK_AND_WHITE_LIST TO "_SYSTEM";

GRANT SELECT ON ROOT_ROOT    TO "_SYSTEM";
GRANT SELECT ON ROOT_TABLES  TO "_SYSTEM";
GRANT SELECT ON ROOT_COLUMNS TO "_SYSTEM";

GRANT DELETE ON INFORMATION_SCHEMA_CATALOG_NAME TO "_SYSTEM";

COMMIT;

ALTER TABLE DEFINITION_SCHEMA.CATALOGS SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.SCHEMATA SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.TABLES SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.DATA_TYPE_DESCRIPTOR SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.VIEWS SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.COLUMNS SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.TABLE_CONSTRAINTS SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.KEY_COLUMN_USAGE SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.CHECK_CONSTRAINTS SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.TABLE_PRIVILEGES SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.COLUMN_PRIVILEGES SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.INDEXES SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.GROUP_USER_USAGE SET PRESERVE TRUE;
ALTER TABLE DEFINITION_SCHEMA.LS_INDEXES SET PRESERVE TRUE;

COMMIT;
