#ifndef FBCBlobHandle__H
#define FBCBlobHandle__H

/*!
@header FBCBlobHandle
 @discussion FBCBlobHandle is a client-side pointer to a large object (BLOB or CLOB) in the database server.  Using an FBCBlobHandle and the associated FBCDatabaseConnection, you can retrieve a BLOB or CLOB in its entirety, or use the piece-wise streaming functions to seek through and read portions of the large object content.
*/

/*!
@function fbcbhInitWithHandle
 @discussion Create a new FBCBlobHandle using the string representation of a handle.
 @result The returned FBCBlobHandle should be released by the caller using fbcbhRelease().
 */
FBCBlobHandle* fbcbhInitWithHandle(char* handle);

/*!
@function fbcbhMakeCopy
 @discussion Make a copy of a FBCBlobHandle.
 @result The returned FBCBlobHandle should be released by the caller using fbcbhRelease().
 */
FBCBlobHandle *fbcbhMakeCopy(FBCBlobHandle* self);

/*!
@function fbcbhGetHandle
 @discussion Get the string representation of an FBCBlobHandle.  The "handle" buffer must be at least 28 bytes long (27 bytes for data, one byte for NULL).
 */
void fbcbhGetHandle(FBCBlobHandle* self, char* handle);

/*!
@function fbcbhBlobSize
 @discussion Return the size of the BLOB.
 */
unsigned fbcbhBlobSize(FBCBlobHandle* self);

/*!
@function fbcbhDescription
 @discussion Internal API.
 */
char* fbcbhDescription(FBCBlobHandle* self);

/*!
@function fbcbhRelease
 @discussion Release the FBCBlobHandle and its associated resources.
 */
void fbcbhRelease(FBCBlobHandle* self);

#endif /* FBCBlobHandle__H */
