#ifndef FBCColumnMetaData__H
#define FBCColumnMetaData__H

/*!
@header FBCColumnMetaData
 @discussion FBCColumnMetaData objects are maintained by FBCMetaData.  They are read-only and you shouldn't worry about allocating or releasing them.

 Use the function fbcmdColumnMetaDataAtIndex() to get access to FBCColumnMetaData objects.
*/

/*!
@function fbccmdSchemaName
 @discussion The schema name associated with the column.
 @result The returned string should not be freed by the caller.
 */
const char* fbccmdSchemaName(const FBCColumnMetaData* self);

/*!
@function fbccmdTableName
 @discussion The table name associated with the column.
 @result The returned string should not be freed by the caller.
 */
const char* fbccmdTableName(const FBCColumnMetaData* self);

/*!
@function fbccmdColumnName
 @discussion The name of the column.
 @result The returned string should not be freed by the caller.
 */
const char* fbccmdColumnName(const FBCColumnMetaData* self);

/*!
@function fbccmdLabelName
 @discussion The label of the column.  This can differ from the column name if a column alias is used.
 @result The returned string should not be freed by the caller.
 */
const char* fbccmdLabelName(const FBCColumnMetaData* self);

/*!
@function fbccmdIsNullable
 @discussion Returns True of the column allows NULL.
 */
Bool  fbccmdIsNullable(const FBCColumnMetaData* self);

/*!
@function fbccmdDatatype
 @discussion Return the FBCDatatypeMetaData associated with this column.
 @result The returned FBCDatatypeMetaData should not be freed by the caller.
 */
const FBCDatatypeMetaData* fbccmdDatatype(const FBCColumnMetaData* self);

// Only for internal use, should not be called from outside FBCAccess

/*!
@function fbccmdMetaDataWithComponents
 @discussion Internal API.  Not for public use.
 */
FBCColumnMetaData* fbccmdMetaDataWithComponents(char* str);

/*!
@function fbccmdRelease
 @discussion Release the FBCColumnMetaData instance and all related resources.
 */
void fbccmdRelease(FBCColumnMetaData* self);

#endif /*FBCColumnMetaData__H */
