#ifndef FBCDatatypeMetaData__H
#define FBCDatatypeMetaData__H

/*!
@header FBCDatatypeMetaData
 @discussion FBCDatatypeMetaData objects are maintained by FBCMetaData.  They are read-only and you shouldn't worry about allocating or releasing them.

 Use the methods fbcmdDatatypeMetaDataAtIndex or fbccmdDatatype to get access to FBCDatatypeMetaData objects.
 */

/*!
@function fbcdmdDatatypeCode
 @discussion The internal datatype code.
 */
unsigned fbcdmdDatatypeCode(const FBCDatatypeMetaData* self);

/*!
@function fbcdmdLength
 @discussion The length of the datatype in bytes.
 */
unsigned fbcdmdLength(const FBCDatatypeMetaData* self);

/*!
@function fbcdmdPrecision
 @discussion The precision of the datatype.
 */
unsigned fbcdmdPrecision(const FBCDatatypeMetaData* self);

/*!
@function fbcdmdScale
 @discussion The scale of the datatype.
 */
unsigned fbcdmdScale(const FBCDatatypeMetaData* self);

/*!
@function fbcdmdFractionalPrecision
 @discussion The fractional of the datatype (in case of e.g. INTERVAL '...' DAY TO SECOND(6))
 */
unsigned fbcdmdFractionalPrecision(const FBCDatatypeMetaData* self);

/*!
@function fbcdmdDatatypeString
 @discussion The name of the datatype.
 @result The returned string should not be freed by the caller.
 */
const char* fbcdmdDatatypeString(const FBCDatatypeMetaData* self);

/*!
@function fbcdmdFrom
 @discussion The "from" portion of a SQL 92 INTERVAL.
 */
const char* fbcdmdFrom(const FBCDatatypeMetaData* self);

/*!
@function fbcdmdTo
 @discussion The "to" portion of a SQL 92 INTERVAL.
 */
const char* fbcdmdTo(const FBCDatatypeMetaData* self);

// Only for internal use, should not be called from outside FBCAccess

/*!
@function fbcdmdMetaDataWithComponents
 @discussion Internal API.
 */
FBCDatatypeMetaData* fbcdmdMetaDataWithComponents(char* str);

/*!
@function fbcdmdRelease
 @discussion Internal API.
 */
void fbcdmdRelease(FBCDatatypeMetaData* self);

#endif /* FBCDatatypeMetaData__H */
