#ifndef FBCDateSupport__H
#define FBCDateSupport__H


/*!
@function fbcdsDaysInMonth
 @discussion Return the number of days in the given month.
 */
int fbcdsDaysInMonth(unsigned month);

/*!
@function fbcdsIsLeapYear
 @discussion Return true if the given year is a leap year.
 */
Bool fbcdsIsLeapYear(FBUShort year);

/*!
@function fbcdsConvertYearToDays
 @discussion Convert the given year into a days offset.
 */
int fbcdsConvertYearToDays(FBUShort year);

/*!
@function fbcdsConvertDateToYearMonthDay
 @discussion Extract the year, month, and day components of the given year.
 */
void fbcdsConvertDateToYearMonthDay(double date, int* year, int* month, int* day);

/*!
@function fbcdsConvertTimeToHourMinuteSeconds
 @discussion Extract the hour, minute, and seconds from the given time.
 */
void fbcdsConvertTimeToHourMinuteSeconds(double time, int* hour, int* minute, double* seconds);

/*!
@function fbcdsConvertTimestampToDateAndTime
 @discussion Extract the date and time components from a SQL 92 TIMESTAMP.
 */
void fbcdsConvertTimestampToDateAndTime(double timestamp, double* datePart, double* timePart);

/*!
@function fbcdsBuildTimeString
 @discussion Build a SQL 92 compliant TIME string from the given components.  The "str" buffer must be large enough to hold the result.  See the SQL 92 standard for formatting details.
 */
int fbcdsBuildTimeString(char* str, double time, int tzOffset);

/*!
@function fbcdsBuildTimeTZString
 @discussion Build a SQL 92 compliant TIME WITH TIME ZONE string from the given components.  The "str" buffer must be large enough to hold the result.  See the SQL 92 standard for formatting details.
 */
int fbcdsBuildTimeTZString(char* str, double time, unsigned tzSign, int tzValue);

/*!
@function fbcdsBuildTimestampString
 @discussion Build a SQL 92 compliant TIMESTAMP string from the given components.  The "str" buffer must be large enough to hold the result.  See the SQL 92 standard for formatting details.
 */
int fbcdsBuildTimestampString(char* str, double timestamp, FBCTimeZone* timeZone);
int fbcdsBuildTimestampStringP(char* str, double timestamp, FBCTimeZone* timeZone, int precision);

/*!
@function fbcdsBuildTimestampTZString
 @discussion Build a SQL 92 compliant TIMESTAMP WITH TIME ZONE string from the given components.  The "str" buffer must be large enough to hold the result.  See the SQL 92 standard for formatting details.
 */
int fbcdsBuildTimestampTZString(char* str, double timestamp, unsigned tzSign, int tzValue );
int fbcdsBuildTimestampTZStringP(char* str, double timestamp, unsigned tzSign, int tzValue, int precision );

#endif /* FBCDateSupport__H */
