#ifndef FBCErrorMetaData__H
#define FBCErrorMetaData__H

/*!
@header FBCErrorMetaData
 @discussion FBCErrorMetaData objects provide error information about a failed database operation.

 FBCErrorMetaData objects are allocated by the fbcdcErrorMetaData() function in the FBCDatabaseConnection suite.  Once they're not longer necessary, the caller should release them using fbcemdRelease().
*/

/*!
@function fbcemdAllErrorMessages
 @discussion Return all of the error messages associated with the previous operation.
 @result The returned string should be freed by the caller.
 */
char* fbcemdAllErrorMessages(FBCErrorMetaData* self);

/*!
@function fbcemdErrorCount
 @discussion Return the error count.
 */
unsigned fbcemdErrorCount(FBCErrorMetaData* self);

/*!
@function fbcemdErrorCodeAtIndex
 @discussion Returns the error code at the given index.  The codes are defined in <install root>/FrontBase/.BootstrapFiles/FBSQLErrors.array
 */
unsigned fbcemdErrorCodeAtIndex(FBCErrorMetaData* self, unsigned errorIndex);

/*!
@function fbcemdErrorMessageAtIndex
 @discussion Returns the error message at the given index.
 @result The returned string should be freed by the caller.
 */
char* fbcemdErrorMessageAtIndex(FBCErrorMetaData* self, unsigned errorIndex);

/*!
@function fbcemdErrorKindAtIndex
 @discussion Returns the kind of error at the given index.  Error kinds are, "Syntax Error", "Semantic Error", "Completion condition", "Exception", "Connection error", and "License error".
 */
const char* fbcemdErrorKindAtIndex(FBCErrorMetaData* self, unsigned errorIndex);

/*!
@function fbcemdRelease
 @discussion Releases the resources associated with an FBCErrorMetaData.
 */
void fbcemdRelease(FBCErrorMetaData* self);

/*!
@function fbcemdMetaDataFromString
 @discussion Internal API.
 */
FBCErrorMetaData* fbcemdMetaDataFromString(FBCDatabaseConnection* fbdc, char* str);

#endif /* FBCErrorMetaData__H */
